----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    13:33:04 01/23/2017 
-- Design Name: 
-- Module Name:    POWER_CONTROL - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

Library UNISIM;
use UNISIM.vcomponents.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;


		
entity POWER_CONTROL is
	port(
		CLK_10M			: in 		std_logic;	
		test_port_o		: out		std_logic_vector(7 downto 0);
		RST_PWR			: in 		std_logic;	
		
		PG_LED			: out		std_logic;
		
		VCC5_PG			: in 		std_logic;
		VCC2V5_PG		: in 		std_logic;
		VCC3V3_AUX_PG	: in 		std_logic;
		VCC1V8_PG		: in 		std_logic;	
		VCC1V5_PG		: in 		std_logic;	
		VCC0V75_PG		: in		std_logic;
		UCD9222_PG1		: in 		std_logic;	
		UCD9222_PG2		: in 		std_logic;			
		UCD9222_PG		: in 		std_logic;	
		
		VCC5V_EN			: out		std_logic;	
		VCC2V5_EN		: out		std_logic;
		VCC1V8_EN1		: out		std_logic;  
		UCD9222_ENA1	: out		std_logic;  
		UCD9222_ENA2	: out		std_logic;  		
		VCC1V5_EN		: out		std_logic;	
		VCC0V75_EN		: out		std_logic;
		CDCE_POWER_p 	: out		std_logic;
		cdce_plllock_p : in 		std_logic


	
	);


end POWER_CONTROL;

architecture Behavioral of POWER_CONTROL is

	
	signal 		spower_count24	: std_logic_vector(23 downto 0)	:= x"000000";
	signal 		ssys_pow_on		: std_logic := '0';
	

	
-- contant waiting time for power on 	
	constant		cturnon1		: std_logic_vector( 23 downto 0) := x"0186A0"; -- 10msec
	constant		cturnon2		: std_logic_vector( 23 downto 0) := x"0249F0"; -- 15msec
	constant		cturnon3		: std_logic_vector( 23 downto 0) := x"030D40"; -- 20msec
	constant		cturnon4		: std_logic_vector( 23 downto 0) := x"03D090"; -- 25msec
	constant		cturnon5		: std_logic_vector( 23 downto 0) := x"055730"; -- 35msec
	constant		cturnon6		: std_logic_vector( 23 downto 0) := x"061A80"; -- 40msec
	constant		cturnon7		: std_logic_vector( 23 downto 0) := x"06DDD0"; -- 45msec
	constant		cturnon8		: std_logic_vector( 23 downto 0) := x"07A120"; -- 50msec

	
begin
-- Power sequence VCC_5V_EN ->(10ms) VCC2V5_EN ->(5ms) UCD9222_EN1 ->(5ms) UCD9222_EN2 ->(5ms) VCC1V8_EN ->(10ms) VCC1V5_EN ->(5ms) VCC0V75_EN

	--test_port_o <= (VCC5_PG, VCC2V5_PG,VCC1V8_PG, VCC1V5_PG, VCC0V75_PG, UCD9222_PG1, UCD9222_PG2, UCD9222_PG);
	--test_port_o <= (VCC5V_EN, VCC2V5_EN,VCC1V8_EN1, VCC1V5_EN, VCC0V75_EN, UCD9222_ENA1, UCD9222_ENA2, UCD9222_ENA2);
	
	
	store_buff: process(CLK_10M)
	begin
		if(CLK_10M'event and CLK_10M = '1') then
			test_port_o <= (VCC5_PG, VCC2V5_PG,VCC1V8_PG, VCC1V5_PG, VCC0V75_PG, UCD9222_PG1, UCD9222_PG2, cdce_plllock_p);
		end if;	
	end process;



	

	power_count24:process(CLK_10M,RST_PWR)
	begin
		if(RST_PWR = '1') then
			 if(CLK_10M'event and CLK_10M = '1') then

					if(spower_count24 <  x"FFFFFE") then
							spower_count24 <= spower_count24 + 1;
					end if;
			 end if;	
		else
			spower_count24 <= x"000000";
		end if;	
	end process;
	
	

		
	power_turnon:process(CLK_10M, spower_count24,RST_PWR)
	begin
		if(RST_PWR = '1') then
			if(CLK_10M'event and CLK_10M = '1') then
				if(spower_count24 = cturnon1) then
						VCC5V_EN <= '1';
					elsif(spower_count24 = cturnon2) then
						VCC2V5_EN <= '1';
					elsif(spower_count24 = cturnon3) then
						UCD9222_ENA1 <= '1';
					elsif(spower_count24 = cturnon4) then
						UCD9222_ENA2 <= '1';
					elsif(spower_count24 = cturnon5) then
						VCC1V8_EN1 <= '1';
					elsif(spower_count24 = cturnon6) then
						VCC1V5_EN <= '1';
					elsif(spower_count24 = cturnon7) then					
						VCC0V75_EN <= '1';
					elsif(spower_count24 = cturnon8) then	
						ssys_pow_on <= '1';
						CDCE_POWER_p <= '1';
				end if;
			end if;
		else
			VCC5V_EN 		<= '0';
			VCC2V5_EN 		<= '0';
			VCC1V8_EN1 		<= '0';
			UCD9222_ENA1 	<= '0';
			UCD9222_ENA2 	<= '0';
			VCC1V5_EN 		<= '0';
			VCC0V75_EN 		<= '0';	
			ssys_pow_on 	<= '0';
			CDCE_POWER_p 	<= '0';
		
		end if;
	end process;
	
		
	check_power:process(CLK_10M,ssys_pow_on )
	begin
		if(CLK_10M'event and CLK_10M = '1') then
			if(ssys_pow_on = '1') then
				PG_LED <= VCC5_PG and VCC2V5_PG and VCC3V3_AUX_PG and VCC1V5_PG and VCC0V75_PG and VCC1V8_PG;-- and UCD9222_PG1 and UCD9222_PG2 and UCD9222_PG;
			else
				PG_LED <= '0';
			end if;
		end if;
	
	end process;
	

end Behavioral;

